@@ -28,16 +28,17 @@ from __future__ import absolute_import
 
 import gevent
 from gevent.server import StreamServer
-from gevent import monkey; monkey.patch_all()
-from dns import resolver, reversename
-from dns.exception import DNSException
+from dns import reversename
+from dns.resolver import NXDOMAIN
 
+from slimta import logging
 from slimta.envelope import Envelope
 from slimta.smtp.server import Server
 from slimta.smtp.reply import unknown_command, bad_sequence
 from slimta.smtp import ConnectionLost, MessageTooBig
 from slimta.queue import QueueError
 from slimta.relay import RelayError
+from slimta.util import dns_resolver
 from . import EdgeServer
 
 __all__ = ['SmtpEdge', 'SmtpValidators']
@@ -50,30 +51,36 @@ class SmtpValidators(object):
     the `reply` argument untouched will return the default, successful reply
     from the command.
 
-    * ``handle_banner(reply, address)``: Validate connecting address before
+    - ``handle_banner(reply, address)``: Validate connecting address before
       sending the SMTP banner.
-    * ``handle_ehlo(reply, ehlo_as)``: Validate the EHLO string.
-    * ``handle_helo(reply, helo_as)``: Validate the HELO string.
-    * ``handle_mail(reply, sender)``: Validate the sender address.
-    * ``handle_rcpt(reply, recipient)``: Validate one recipient address.
-    * ``handle_data(reply)``: Any remaining validation before accepting data.
-    * ``handle_rset(reply)``: Called before replying to an RSET command.
-    * ``handle_tls()``: Called after a successful TLS handshake. This may be at
+    - ``handle_ehlo(reply, ehlo_as)``: Validate the EHLO string.
+    - ``handle_helo(reply, helo_as)``: Validate the HELO string.
+    - ``handle_mail(reply, sender, params)``: Validate the sender address.
+    - ``handle_rcpt(reply, recipient, params)``: Validate one recipient address.
+    - ``handle_data(reply)``: Any remaining validation before receiving data.
+    - ``handle_have_data(reply, data)``: Validate the received message data.
+    - ``handle_rset(reply)``: Called before replying to an RSET command.
+    - ``handle_tls()``: Called after a successful TLS handshake. This may be at
       the beginning of the session or after a `STARTTLS` command.
 
+    :param session: When sub-classes are instantiated, instances are passed
+                    this object, stored and described in :attr:`session` below,
+                    that have useful information about the current session.
+
     """
 
     def __init__(self, session):
-        #: This object has a set of attributes that may be useful in validation:
-        #: 
-        #:  * ``address``: The address tuple of the connecting client.
-        #:  * ``extended_smtp``: The client used EHLO instead of HELO.
-        #:  * ``security``: Security of connection, ``None`` or ``'TLS'``.
-        #:  * ``ehlo_as``: The EHLO or HELO string given by the client.
-        #:  * ``auth_result``: The authentication result returned by |Auth|
+        #: This instance attribute is an object that has its own set of
+        #: attributes which may be useful in validation:
+        #:
+        #:  - ``address``: The address tuple of the connecting client.
+        #:  - ``extended_smtp``: The client used EHLO instead of HELO.
+        #:  - ``security``: Security of connection, ``None`` or ``'TLS'``.
+        #:  - ``ehlo_as``: The EHLO or HELO string given by the client.
+        #:  - ``auth_result``: The authentication result returned by |Auth|
         #:                     after successful authentication, or ``None`` if
         #:                     the client has not authenticated.
-        #:  * ``envelope``: The |Envelope| being pieced together to send by the
+        #:  - ``envelope``: The |Envelope| being pieced together to send by the
         #:                  connecting client.
         self.session = session
 
@@ -109,15 +116,18 @@ class SmtpSession(object):
         return proto
 
     def _ptr_lookup(self):
-        ptraddr = reversename.from_address(self.address[0])
         try:
-            answers = resolver.query(ptraddr, 'PTR')
-        except DNSException:
-            answers = []
-        try:
-            self.reverse_address = str(answers[0])
-        except IndexError:
-            pass
+            ptraddr = reversename.from_address(self.address[0])
+            try:
+                answers = dns_resolver.query(ptraddr, 'PTR')
+            except NXDOMAIN:
+                answers = []
+            try:
+                self.reverse_address = str(answers[0])
+            except IndexError:
+                pass
+        except Exception:
+            logging.log_exception(__name__, query=self.address[0])
 
     def _trigger_ptr_lookup(self):
         self._ptr_lookup_thread = gevent.spawn(self._ptr_lookup)
@@ -149,13 +159,21 @@ class SmtpSession(object):
     def RSET(self, reply):
         self.envelope = None
 
-    def MAIL(self, reply, address):
-        self._call_validator('mail', reply, address)
+    def MAIL(self, reply, address, params):
+        try:
+            self._call_validator('mail', reply, address, params)
+        except TypeError:
+            # XXX: Temporary for backwards-compatibility.
+            self._call_validator('mail', reply, address)
         if reply.code == '250':
             self.envelope = Envelope(sender=address)
 
-    def RCPT(self, reply, address):
-        self._call_validator('rcpt', reply, address)
+    def RCPT(self, reply, address, params):
+        try:
+            self._call_validator('rcpt', reply, address, params)
+        except TypeError:
+            # XXX: Temporary for backwards-compatibility.
+            self._call_validator('rcpt', reply, address)
         if reply.code == '250':
             self.envelope.recipients.append(address)
 
@@ -170,10 +188,15 @@ class SmtpSession(object):
         elif err:
             raise err
 
+        self._call_validator('have_data', reply, data)
+        if reply.code != '250':
+            return
+
         self.envelope.client['ip'] = self.address[0]
         self.envelope.client['host'] = self.reverse_address
         self.envelope.client['name'] = self.ehlo_as
         self.envelope.client['protocol'] = self.protocol
+        self.envelope.client['auth'] = self.auth_result
         if hasattr(self, '_ptr_lookup_thread'):
             self._ptr_lookup_thread.kill(block=False)
 
@@ -196,9 +219,10 @@ class SmtpEdge(EdgeServer):
     """Class that uses :mod:`slimta.smtp.server` to implement an edge
     service to receive messages.
 
-    :param listener: ``(ip, port)`` tuple to listen on, as described in |Edge|.
+    :param listener: ``(ip, port)`` tuple to listen on, as described in
+                     |EdgeServer|.
     :param queue: |Queue| object for handing off messages, as described in
-                  :meth:`Edge.handoff()`.
+                  :meth:`~slimta.edge.Edge.handoff()`.
     :param pool: Optional greenlet pool, as described in |Edge|.
     :param max_size: Maximum size of incoming messages.
     :param validator_class: :class:`SmtpValidators` sub-class to validate
@@ -220,10 +244,10 @@ class SmtpEdge(EdgeServer):
     """
 
     def __init__(self, listener, queue, pool=None, max_size=None,
-                       validator_class=None, auth_class=None,
-                       tls=None, tls_immediately=False,
-                       command_timeout=None, data_timeout=None,
-                       hostname=None):
+                 validator_class=None, auth_class=None,
+                 tls=None, tls_immediately=False,
+                 command_timeout=None, data_timeout=None,
+                 hostname=None):
         super(SmtpEdge, self).__init__(listener, queue, pool, hostname)
         self.max_size = max_size
         self.command_timeout = command_timeout
