@@ -4,7 +4,7 @@ import unittest
 from mox import MoxTestBase, IsA
 from gevent import socket, ssl
 
-from slimta.http import HTTPConnection, HTTPSConnection
+from slimta.http import HTTPConnection, HTTPSConnection, get_connection
 
 
 class TestHTTPConnection(MoxTestBase):
@@ -28,12 +28,12 @@ class TestHTTPSConnection(MoxTestBase):
         ssl.SSLSocket(9, var='val').AndReturn(sslsock)
         sslsock.do_handshake()
         self.mox.ReplayAll()
-        conn = HTTPSConnection('testhost', 8025, True, 7, 8, tls={'var': 'val'})
+        conn = HTTPSConnection('testhost', 8025, {'var': 'val'}, True, 7, 8)
         conn.connect()
         self.assertEqual(sslsock, conn.sock)
 
     def test_close(self):
-        conn = HTTPSConnection('testhost', 8025, True, 7, 8, tls={'var': 'val'})
+        conn = HTTPSConnection('testhost', 8025, {'var': 'val'}, True, 7, 8)
         conn.sock = self.mox.CreateMockAnything()
         conn.sock.unwrap()
         conn.sock.close()
@@ -41,4 +41,13 @@ class TestHTTPSConnection(MoxTestBase):
         conn.close()
 
 
+class TestGetConnection(MoxTestBase):
+
+    def test_get_connection(self):
+        conn = get_connection('http://localhost')
+        self.assertIsInstance(conn, HTTPConnection)
+        conn = get_connection('https://localhost')
+        self.assertIsInstance(conn, HTTPSConnection)
+
+
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
